@@ -45,6 +45,7 @@ pub mod jsonb;
 pub mod jsondiff;
 pub mod misc;
 pub mod row;
+pub mod time;
 pub mod value;
 
 pub struct BinlogCtx<'a> {
@@ -203,11 +204,11 @@ impl EventStreamReader {
 
         if event_type == EventType::FORMAT_DESCRIPTION_EVENT as u8 {
             // we'll redefine fde with an actual one
-            let fde = event.read_event::<FormatDescriptionEvent>()?;
+            let fde = event.read_event::<FormatDescriptionEvent<'_>>()?;
             self.fde = fde.into_owned().with_footer(event.footer());
         } else if event_type == EventType::TABLE_MAP_EVENT as u8 {
             // we'll maintain known table maps
-            let tme = event.read_event::<TableMapEvent>()?;
+            let tme = event.read_event::<TableMapEvent<'_>>()?;
             self.table_map.insert(tme.table_id(), tme.into_owned());
         } else if event_type == EventType::ROTATE_EVENT as u8 {
             // we'll keep table map size within reasonlable bounds
@@ -215,7 +216,7 @@ impl EventStreamReader {
             // TODO: This value is arbitrary
             const TABLE_MAP_MAX_SIZE: usize = 64;
 
-            let re = event.read_event::<RotateEvent>()?;
+            let re = event.read_event::<RotateEvent<'_>>()?;
             if !re.is_fake() {
                 self.table_map.clear();
                 self.table_map.shrink_to(TABLE_MAP_MAX_SIZE);
@@ -285,7 +286,8 @@ impl ColumnType {
             | Self::MYSQL_TYPE_TIME2
             | Self::MYSQL_TYPE_DATETIME2
             | Self::MYSQL_TYPE_TIMESTAMP2
-            | Self::MYSQL_TYPE_JSON => ptr.get(..1).map(|x| (x, 1)),
+            | Self::MYSQL_TYPE_JSON
+            | Self::MYSQL_TYPE_VECTOR => ptr.get(..1).map(|x| (x, 1)),
             Self::MYSQL_TYPE_VARCHAR => {
                 if is_array {
                     ptr.get(..3).map(|x| (x, 3))
@@ -302,7 +304,21 @@ impl ColumnType {
                 .ok()?
                 .get_metadata(ptr.get(1..)?, true)
                 .map(|(x, n)| (x, n + 1)),
-            _ => Some((&[], 0)),
+            Self::MYSQL_TYPE_DECIMAL
+            | Self::MYSQL_TYPE_TINY
+            | Self::MYSQL_TYPE_SHORT
+            | Self::MYSQL_TYPE_LONG
+            | Self::MYSQL_TYPE_NULL
+            | Self::MYSQL_TYPE_TIMESTAMP
+            | Self::MYSQL_TYPE_LONGLONG
+            | Self::MYSQL_TYPE_INT24
+            | Self::MYSQL_TYPE_DATE
+            | Self::MYSQL_TYPE_TIME
+            | Self::MYSQL_TYPE_DATETIME
+            | Self::MYSQL_TYPE_YEAR
+            | Self::MYSQL_TYPE_NEWDATE
+            | Self::MYSQL_TYPE_UNKNOWN
+            | Self::MYSQL_TYPE_VAR_STRING => Some((&[], 0)),
         }
     }
 }
@@ -311,21 +327,26 @@ impl ColumnType {
 mod tests {
     use std::{
         collections::HashMap,
+        convert::TryFrom,
         io,
         iter::{once, repeat},
     };
 
     use super::{
+        BinlogFile, BinlogFileHeader, BinlogVersion,
         consts::{EventFlags, EventType},
         events::{BinlogEventHeader, EventData, GtidEvent},
-        BinlogFile, BinlogFileHeader, BinlogVersion,
     };
 
     use crate::{
-        binlog::{events::RowsEventData, value::BinlogValue},
+        binlog::{
+            events::{OptionalMetadataField, RowsEventData},
+            value::BinlogValue,
+        },
         collations::CollationId,
         constants::ColumnFlags,
         proto::MySerialize,
+        row::convert::from_row,
         value::Value,
     };
 
@@ -719,10 +740,12 @@ mod tests {
             let file_data = std::fs::read(dbg!(&file_path))?;
             let mut binlog_file = BinlogFile::new(BinlogVersion::Version4, &file_data[..])?;
 
+            let mut i = 0;
             let mut ev_pos = 4;
             let mut table_map_events = HashMap::new();
 
             while let Some(ev) = binlog_file.next() {
+                i += 1;
                 let ev = ev?;
                 let _ = dbg!(ev.header().event_type());
                 let ev_end = ev_pos + ev.header().event_size() as usize;
@@ -832,6 +855,148 @@ mod tests {
                     }
                 }
 
+                if file_path.file_name().unwrap() == "json-opaque.binlog" {
+                    let event_data = ev.read_data().unwrap();
+
+                    /// Extracts first column of the binlog row after-image as a Jsonb::Value
+                    /// then parses it into the structured representation and compares with
+                    /// the expected value.
+                    macro_rules! extract_cmp {
+                        ($row:expr_2021, $expected:tt) => {
+                            let mut after = $row.1.unwrap().unwrap();
+                            let a = dbg!(after.pop().unwrap());
+                            let super::value::BinlogValue::Jsonb(a) = a else {
+                                panic!("BinlogValue::Jsonb(_) expected");
+                            };
+                            assert_eq!(
+                                serde_json::json!($expected),
+                                serde_json::Value::from(a.parse().unwrap())
+                            );
+                        };
+                    }
+
+                    match event_data {
+                        Some(EventData::RowsEvent(ev)) if i == 10 => {
+                            let table_map_event =
+                                binlog_file.reader().get_tme(ev.table_id()).unwrap();
+                            let mut rows = ev.rows(table_map_event);
+                            extract_cmp!(rows.next().unwrap().unwrap(), {"a": "base64:type15:VQ=="});
+                        }
+                        Some(EventData::RowsEvent(ev)) if i == 12 => {
+                            let table_map_event =
+                                binlog_file.reader().get_tme(ev.table_id()).unwrap();
+                            let mut rows = ev.rows(table_map_event);
+                            extract_cmp!(rows.next().unwrap().unwrap(), {"b": "2012-03-18"});
+                        }
+                        Some(EventData::RowsEvent(ev)) if i == 14 => {
+                            let table_map_event =
+                                binlog_file.reader().get_tme(ev.table_id()).unwrap();
+                            let mut rows = ev.rows(table_map_event);
+                            extract_cmp!(rows.next().unwrap().unwrap(), {"c": "2012-03-18 11:30:45.000000"});
+                        }
+                        Some(EventData::RowsEvent(ev)) if i == 16 => {
+                            let table_map_event =
+                                binlog_file.reader().get_tme(ev.table_id()).unwrap();
+                            let mut rows = ev.rows(table_map_event);
+                            extract_cmp!(rows.next().unwrap().unwrap(), {"c": "87:31:46.654321"});
+                        }
+                        Some(EventData::RowsEvent(ev)) if i == 18 => {
+                            let table_map_event =
+                                binlog_file.reader().get_tme(ev.table_id()).unwrap();
+                            let mut rows = ev.rows(table_map_event);
+                            extract_cmp!(rows.next().unwrap().unwrap(), {"d": "123.456"});
+                        }
+                        Some(EventData::RowsEvent(ev)) if i == 20 => {
+                            let table_map_event =
+                                binlog_file.reader().get_tme(ev.table_id()).unwrap();
+                            let mut rows = ev.rows(table_map_event);
+                            extract_cmp!(rows.next().unwrap().unwrap(), {"e": "9.00"});
+                        }
+                        Some(EventData::RowsEvent(ev)) if i == 22 => {
+                            let table_map_event =
+                                binlog_file.reader().get_tme(ev.table_id()).unwrap();
+                            let mut rows = ev.rows(table_map_event);
+                            extract_cmp!(rows.next().unwrap().unwrap(), {"e": [0, 1, true, false]});
+                        }
+                        Some(EventData::RowsEvent(ev)) if i == 24 => {
+                            let table_map_event =
+                                binlog_file.reader().get_tme(ev.table_id()).unwrap();
+                            let mut rows = ev.rows(table_map_event);
+                            extract_cmp!(rows.next().unwrap().unwrap(), {"e": null});
+                        }
+                        Some(EventData::RowsEvent(ev)) => {
+                            panic!("no more events expected i={}, {:?}", i, ev);
+                        }
+                        _ => (),
+                    }
+                }
+
+                if file_path.file_name().unwrap() == "vector.binlog" {
+                    let event_data = ev.read_data().unwrap();
+                    match event_data {
+                        Some(EventData::TableMapEvent(ev)) => {
+                            let optional_meta = ev.iter_optional_meta();
+                            match ev.table_name().as_ref() {
+                                "foo" => {
+                                    for meta in optional_meta {
+                                        match meta.unwrap() {
+                                            OptionalMetadataField::Dimensionality(x) => assert_eq!(
+                                                x.iter_dimensionalities()
+                                                    .collect::<Result<Vec<_>, _>>()
+                                                    .unwrap(),
+                                                vec![3],
+                                            ),
+                                            _ => (),
+                                        }
+                                    }
+                                }
+                                "bar" => {
+                                    for meta in optional_meta {
+                                        match meta.unwrap() {
+                                            OptionalMetadataField::Dimensionality(x) => assert_eq!(
+                                                x.iter_dimensionalities()
+                                                    .collect::<Result<Vec<_>, _>>()
+                                                    .unwrap(),
+                                                vec![2, 4],
+                                            ),
+                                            _ => (),
+                                        }
+                                    }
+                                }
+                                _ => (),
+                            }
+                        }
+                        Some(EventData::RowsEvent(ev)) if i == 12 => {
+                            let table_map_event =
+                                binlog_file.reader().get_tme(ev.table_id()).unwrap();
+                            let mut rows = ev.rows(table_map_event);
+
+                            let (None, Some(after)) = rows.next().unwrap().unwrap() else {
+                                panic!("Unexpected data");
+                            };
+                            let (id, vector_column): (u8, Vec<u8>) =
+                                from_row(crate::Row::try_from(after).unwrap());
+                            assert_eq!(id, 1);
+                            assert_eq!(
+                                vector_column,
+                                vec![205, 204, 140, 63, 205, 204, 12, 64, 51, 51, 83, 64]
+                            );
+
+                            let (None, Some(after)) = rows.next().unwrap().unwrap() else {
+                                panic!("Unexpected data");
+                            };
+                            let (id, vector_column): (u8, Vec<u8>) =
+                                from_row(crate::Row::try_from(after).unwrap());
+                            assert_eq!(id, 2);
+                            assert_eq!(
+                                vector_column,
+                                vec![0, 0, 128, 63, 0, 0, 128, 191, 0, 0, 0, 0]
+                            );
+                        }
+                        _ => (),
+                    }
+                }
+
                 if file_path.file_name().unwrap() == "mysql-enum-string-set.000001" {
                     if let Some(EventData::RowsEvent(data)) = ev.read_data().unwrap() {
                         let table_map_event =
@@ -992,6 +1157,59 @@ mod tests {
                     assert_eq!(output, ev.data());
                 }
 
+                // https://github.com/blackbeam/rust_mysql_common/issues/162
+                if file_path.file_name().unwrap() == "minimal_row_metadata.000001" {
+                    let event_data = ev.read_data().unwrap();
+                    match event_data {
+                        Some(EventData::RowsEvent(ev)) => {
+                            let table_map_event =
+                                binlog_file.reader().get_tme(ev.table_id()).unwrap();
+                            let row = ev.rows(table_map_event).next().unwrap().unwrap();
+                            let after_image = row.1.unwrap();
+                            after_image
+                                .columns()
+                                .iter()
+                                .enumerate()
+                                .for_each(|(i, column)| {
+                                    if column.name_str() == "@2" {
+                                        assert_eq!(
+                                            column.character_set(),
+                                            CollationId::UTF8MB4_0900_AI_CI as u16
+                                        );
+                                    } else if column.name_str() == "@4" {
+                                        match after_image.as_ref(i).unwrap() {
+                                            BinlogValue::Value(val) => {
+                                                assert_eq!(val, &Value::Int(3230202323));
+                                            }
+                                            _ => panic!("Expected a value"),
+                                        }
+                                    }
+                                });
+                        }
+                        _ => (),
+                    }
+                }
+
+                if file_path.file_name().unwrap() == "time_issue.000001" {
+                    let event_data = ev.read_data().unwrap();
+                    match event_data {
+                        Some(EventData::RowsEvent(ev)) => {
+                            let table_map_event =
+                                binlog_file.reader().get_tme(ev.table_id()).unwrap();
+                            let row = ev.rows(table_map_event).next().unwrap().unwrap();
+                            let after_image = row.1.unwrap();
+                            after_image.columns().iter().enumerate().for_each(|(i, _)| {
+                                match after_image.as_ref(i).unwrap() {
+                                    BinlogValue::Value(val) => {
+                                        assert_eq!(val, &Value::Time(true, 21, 3, 48, 27, 0));
+                                    }
+                                    _ => panic!("Expected a value"),
+                                }
+                            });
+                        }
+                        _ => (),
+                    }
+                }
                 ev_pos = ev_end;
             }
         }
