@@ -17,7 +17,7 @@ pub static UTF8MB4_GENERAL_CI: u16 = 45;
 
 my_bitflags! {
     StatusFlags,
-    #[error("Unknown flags in the raw value of StatusFlags (raw={:b})", _0)]
+    #[error("Unknown flags in the raw value of StatusFlags (raw={0:b})")]
     UnknownStatusFlags,
     u16,
 
@@ -80,7 +80,7 @@ my_bitflags! {
 
 my_bitflags! {
     CapabilityFlags,
-    #[error("Unknown flags in the raw value of CapabilityFlags (raw={:b})", _0)]
+    #[error("Unknown flags in the raw value of CapabilityFlags (raw={0:b})")]
     UnknownCapabilityFlags,
     u32,
 
@@ -362,7 +362,7 @@ my_bitflags! {
 
 my_bitflags! {
     CursorType,
-    #[error("Unknown flags in the raw value of CursorType (raw={:b})", _0)]
+    #[error("Unknown flags in the raw value of CursorType (raw={0:b})")]
     UnknownCursorType,
     u8,
 
@@ -378,7 +378,7 @@ my_bitflags! {
 
 my_bitflags! {
     StmtExecuteParamsFlags,
-    #[error("Unknown flags in the raw value of StmtExecuteParamsFlags (raw={:b})", _0)]
+    #[error("Unknown flags in the raw value of StmtExecuteParamsFlags (raw={0:b})")]
     UnknownStmtExecuteParamsFlags,
     u8,
 
@@ -391,7 +391,7 @@ my_bitflags! {
 
 my_bitflags! {
     StmtExecuteParamFlags,
-    #[error("Unknown flags in the raw value of StmtExecuteParamFlags (raw={:b})", _0)]
+    #[error("Unknown flags in the raw value of StmtExecuteParamFlags (raw={0:b})")]
     UnknownStmtExecuteParamFlags,
     u8,
 
@@ -404,7 +404,7 @@ my_bitflags! {
 
 my_bitflags! {
     ColumnFlags,
-    #[error("Unknown flags in the raw value of ColumnFlags (raw={:b})", _0)]
+    #[error("Unknown flags in the raw value of ColumnFlags (raw={0:b})")]
     UnknownColumnFlags,
     u16,
 
@@ -615,6 +615,7 @@ pub enum ColumnType {
     MYSQL_TYPE_DATETIME2,
     MYSQL_TYPE_TIME2,
     MYSQL_TYPE_TYPED_ARRAY, // Used for replication only
+    MYSQL_TYPE_VECTOR = 242,
     MYSQL_TYPE_UNKNOWN = 243,
     MYSQL_TYPE_JSON = 245,
     MYSQL_TYPE_NEWDECIMAL = 246,
@@ -671,6 +672,10 @@ impl ColumnType {
     pub fn is_geometry_type(&self) -> bool {
         matches!(self, ColumnType::MYSQL_TYPE_GEOMETRY)
     }
+
+    pub fn is_vector_type(&self) -> bool {
+        matches!(self, ColumnType::MYSQL_TYPE_VECTOR)
+    }
 }
 
 impl TryFrom<u8> for ColumnType {
@@ -698,6 +703,7 @@ impl TryFrom<u8> for ColumnType {
             0x12_u8 => Ok(ColumnType::MYSQL_TYPE_DATETIME2),
             0x13_u8 => Ok(ColumnType::MYSQL_TYPE_TIME2),
             0x14_u8 => Ok(ColumnType::MYSQL_TYPE_TYPED_ARRAY),
+            0xf2_u8 => Ok(ColumnType::MYSQL_TYPE_VECTOR),
             0xf3_u8 => Ok(ColumnType::MYSQL_TYPE_UNKNOWN),
             0xf5_u8 => Ok(ColumnType::MYSQL_TYPE_JSON),
             0xf6_u8 => Ok(ColumnType::MYSQL_TYPE_NEWDECIMAL),
@@ -723,7 +729,7 @@ impl From<ColumnType> for u8 {
 
 my_bitflags! {
     Flags2,
-    #[error("Unknown flags in the raw value of Flags2 (raw={:b})", _0)]
+    #[error("Unknown flags in the raw value of Flags2 (raw={0:b})")]
     UnknownFlags2,
     u32,
 
@@ -739,7 +745,7 @@ my_bitflags! {
 
 my_bitflags! {
     SqlMode,
-    #[error("Unknown flags in the raw value of SqlMode (raw={:b})", _0)]
+    #[error("Unknown flags in the raw value of SqlMode (raw={0:b})")]
     UnknownSqlMode,
     u64,
 
