@@ -52,9 +52,7 @@
 //! | `mysql_common::Deserialized<T : DeserializeOwned>` | MySql bytes parsed using `serde_json::from_str` |
 //! | `Option<T: FromValue>`               | Must be used for nullable columns to avoid errors         |
 //! | [`decimal::Decimal`]                 | MySql int, uint or bytes parsed using `Decimal::from_str`.<br>⚠️ Note that this type doesn't support full range of MySql `DECIMAL` type. |
-//! | [`bigdecimal::BigDecimal`] (v0.2.x)  | MySql int, uint, floats or bytes parsed using `BigDecimal::parse_bytes`.<br>⚠️ Note that range of this type is greater than supported by MySql `DECIMAL` type but it'll be serialized anyway. |
-//! | [`bigdecimal::BigDecimal`] (v0.3.x)  | MySql int, uint, floats or bytes parsed using `BigDecimal::parse_bytes`.<br>⚠️ Note that range of this type is greater than supported by MySql `DECIMAL` type but it'll be serialized anyway. |
-//! | [`bigdecimal::BigDecimal`] (v0.4.x)  | MySql int, uint, floats or bytes parsed using `BigDecimal::parse_bytes`.<br>⚠️ Note that range of this type is greater than supported by MySql `DECIMAL` type but it'll be serialized anyway. |
+//! | [`bigdecimal::BigDecimal`]           | MySql int, uint, floats or bytes parsed using `BigDecimal::parse_bytes`.<br>⚠️ Note that range of this type is greater than supported by MySql `DECIMAL` type but it'll be serialized anyway. |
 //! | `num_bigint::{BigInt, BigUint}`      | MySql int, uint or bytes parsed using `_::parse_bytes`.<br>⚠️ Note that range of this type is greater than supported by MySql integer types but it'll be serialized anyway (as decimal bytes string). |
 //!
 //! Also crate provides from-row convertion for the following list of types (see `FromRow` trait):
@@ -68,18 +66,18 @@
 //!
 //! ## Crate features
 //!
-//! | Feature        | Description                                          | Default |
-//! | -------------- | ---------------------------------------------------- | ------- |
-//! | `bigdecimal02` | Enables `bigdecimal` v0.2.x types support            | 🔴      |
-//! | `bigdecimal03` | Enables `bigdecimal` v0.3.x types support            | 🔴      |
-//! | `bigdecimal`   | Enables `bigdecimal` v0.4.x types support            | 🟢      |
-//! | `chrono`       | Enables `chrono` types support                       | 🔴      |
-//! | `rust_decimal` | Enables `rust_decimal` types support                 | 🟢      |
-//! | `time02`       | Enables `time` v0.2.x types support                  | 🔴      |
-//! | `time`         | Enables `time` v0.3.x types support                  | 🟢      |
-//! | `frunk`        | Enables `FromRow` for `frunk::Hlist!` types          | 🟢      |
-//! | `derive`       | Enables [`FromValue` and `FromRow` derive macros][2] | 🟢      |
-//! | `binlog`       | Binlog-related functionality                         | 🟢      |
+//! | Feature          | Description                                          | Default |
+//! | ---------------- | ---------------------------------------------------- | ------- |
+//! | `bigdecimal`     | Enables `bigdecimal` >=0.3.x, <0.5.x types support   | 🔴      |
+//! | `chrono`         | Enables `chrono` types support                       | 🔴      |
+//! | `rust_decimal`   | Enables `rust_decimal` types support                 | 🔴      |
+//! | `time`           | Enables `time` v0.3.x types support                  | 🔴      |
+//! | `frunk`          | Enables `FromRow` for `frunk::Hlist!` types          | 🔴      |
+//! | `derive`         | Enables [`FromValue` and `FromRow` derive macros][2] | 🟢      |
+//! | `binlog`         | Binlog-related functionality                         | 🔴      |
+//! | `client_ed25519` | Enables [ed25519] authentication plugin support      | 🔴      |   
+//!
+//! [ed25519]: https://mariadb.com/kb/en/authentication-plugin-ed25519/
 //!
 //! # Derive Macros
 //!
@@ -261,9 +259,12 @@
 //! *  `#[mysql(rename = "some_name")]` – overrides column name of a field
 //! *  `#[mysql(json)]` - column will be interpreted as a JSON string containing
 //!    a value of a field type
-//! *  `#[mysql(with = path::to::convert_fn)]` – `convert_fn` will be used to deserialize
-//!    a field value (expects a function with a signature that mimics
-//!    `TryFrom<Value, Error=FromValueError>``)
+//! *  `#[mysql(deserialize_with = "some::path")]` – the following function
+//!    will be used to deserialize the field (instead of `FromValue`). Expected signature is
+//!    `fn (Value) -> Result<T, FromValueError>`.
+//! *  `#[mysql(serialize_with = "some::path")]` – the following function
+//!    will be used to serialize the field (instead of `Into<Value>`). Expected signature is
+//!    `fn (T) -> Value`.
 //!
 //! ### Example
 //!
@@ -272,21 +273,45 @@
 //! # use mysql_common::{
 //! #     constants::ColumnType,
 //! #     packets::Column,
+//! #     prelude::FromValue,
 //! #     row::{Row, new_row},
 //! #     row::convert::from_row,
 //! #     value::Value,
+//! #     value::convert::from_value,
+//! #     FromValueError,
 //! # };
+//! use time::{
+//!     macros::{datetime, offset}, OffsetDateTime, PrimitiveDateTime, UtcOffset,
+//! };
 //!
 //! /// Note: the `crate_name` attribute should not be necessary.
 //! #[derive(Debug, PartialEq, Eq, FromRow)]
 //! #[mysql(table_name = "Foos", crate_name = "mysql_common")]
 //! struct Foo {
 //!     id: u64,
+//!     #[mysql(
+//!         serialize_with = "datetime_to_value",
+//!         deserialize_with = "value_to_datetime",
+//!     )]
+//!     ctime: OffsetDateTime,
 //!     #[mysql(json, rename = "def")]
 //!     definition: Bar,
 //!     child: Option<u64>,
 //! }
 //!
+//! fn value_to_datetime(value: Value) -> Result<OffsetDateTime, FromValueError> {
+//!     // assume mysql session timezone has been properly set up
+//!     const OFFSET: UtcOffset = offset!(+3);
+//!
+//!     let primitive = PrimitiveDateTime::from_value_opt(value)?;
+//!     Ok(primitive.assume_offset(OFFSET))
+//! }
+//!
+//! fn datetime_to_value(datetime: OffsetDateTime) -> Value {
+//!     // assume mysql session timezone has been properly set up
+//!     PrimitiveDateTime::new(datetime.date(), datetime.time()).into()
+//! }
+//!
 //! #[derive(Debug, serde::Deserialize, PartialEq, Eq)]
 //! enum Bar {
 //!     Left,
@@ -296,19 +321,25 @@
 //! /// Returns the following row:
 //! ///
 //! /// ```
-//! /// +----+-----------+-------+
-//! /// | id | def       | child |
-//! /// +----+-----------+-------+
-//! /// | 42 | '"Right"' | NULL  |
-//! /// +----+-----------+-------+
+//! /// +----+-----------+-------+------------------------+
+//! /// | id | def       | child | ctime                  |
+//! /// +----+-----------+-------+------------------------+
+//! /// | 42 | '"Right"' | NULL  | '2015-05-15 12:00:00'  |
+//! /// +----+-----------+-------+------------------------+
 //! /// ```
 //! fn get_row() -> Row {
 //!     // ...
-//! #   let values = vec![Value::Int(42), Value::Bytes(b"\"Right\"".as_slice().into()), Value::NULL];
+//! #   let values = vec![
+//! #       Value::Int(42),
+//! #       Value::Bytes(b"\"Right\"".as_slice().into()),
+//! #       Value::NULL,
+//! #       Value::Date(2015, 5, 15, 12, 0, 0, 0),
+//! #   ];
 //! #   let columns = vec![
 //! #       Column::new(ColumnType::MYSQL_TYPE_LONG).with_name(b"id"),
 //! #       Column::new(ColumnType::MYSQL_TYPE_BLOB).with_name(b"def"),
 //! #       Column::new(ColumnType::MYSQL_TYPE_NULL).with_name(b"child"),
+//! #       Column::new(ColumnType::MYSQL_TYPE_STRING).with_name(b"ctime"),
 //! #   ];
 //! #   new_row(values, columns.into_boxed_slice().into())
 //! }
@@ -320,7 +351,15 @@
 //! assert_eq!(Foo::CHILD_FIELD, "child");
 //!
 //! let foo = from_row::<Foo>(get_row());
-//! assert_eq!(foo, Foo { id: 42, definition: Bar::Right, child: None });
+//! assert_eq!(
+//!     foo,
+//!     Foo {
+//!         id: 42,
+//!         definition: Bar::Right,
+//!         child: None,
+//!         ctime: datetime!(2015-05-15 12:00 +3),
+//!     }
+//! );
 //! # }
 //! ```
 //!
@@ -333,7 +372,7 @@
 
 // The `test` feature is required to compile tests.
 // It'll bind test binaries to an official C++ impl of MySql decimals (see build.rs)
-// The idea is to test our rust impl agaist C++ impl.
+// The idea is to test our rust impl against C++ impl.
 #[cfg(all(not(feature = "test"), test))]
 compile_error!("Please invoke `cargo test` with `--features test` flags");
 
@@ -343,12 +382,6 @@ extern crate test;
 #[macro_use]
 pub mod bitflags_ext;
 
-#[cfg(feature = "bigdecimal02")]
-pub use bigdecimal02;
-
-#[cfg(feature = "bigdecimal03")]
-pub use bigdecimal03;
-
 #[cfg(feature = "bigdecimal")]
 pub use bigdecimal;
 
@@ -361,9 +394,6 @@ pub use frunk;
 #[cfg(feature = "rust_decimal")]
 pub use rust_decimal;
 
-#[cfg(feature = "time02")]
-pub use time02;
-
 #[cfg(feature = "time")]
 pub use time;
 
@@ -378,11 +408,11 @@ pub use num_bigint;
 pub use serde;
 pub use serde_json;
 
-pub use value::convert::FromValueError;
 pub use value::Value;
+pub use value::convert::FromValueError;
 
-pub use row::convert::FromRowError;
 pub use row::Row;
+pub use row::convert::FromRowError;
 
 pub use value::json::{Deserialized, Serialized};
 
@@ -397,7 +427,7 @@ pub mod prelude {
     #[doc(inline)]
     pub use mysql_common_derive::FromRow;
 
-    pub use crate::row::{convert::FromRow, ColumnIndex};
+    pub use crate::row::{ColumnIndex, convert::FromRow};
     pub use crate::value::convert::{FromValue, ToValue};
 }
 
@@ -413,7 +443,7 @@ pub mod prelude {
 #[macro_export]
 macro_rules! params {
     () => {};
-    (@to_pair $map:expr, $name:expr => $value:expr) => (
+    (@to_pair $map:expr_2021, $name:expr_2021 => $value:expr_2021) => (
         let entry = $map.entry(std::vec::Vec::<u8>::from($name));
         if let std::collections::hash_map::Entry::Occupied(_) = entry {
             panic!("Redefinition of named parameter `{}'", std::string::String::from_utf8_lossy(entry.key()));
@@ -421,7 +451,7 @@ macro_rules! params {
             entry.or_insert($crate::value::Value::from($value));
         }
     );
-    (@to_pair $map:expr, $name:ident) => (
+    (@to_pair $map:expr_2021, $name:ident) => (
         let entry = $map.entry(stringify!($name).as_bytes().to_vec());
         if let std::collections::hash_map::Entry::Occupied(_) = entry {
             panic!("Redefinition of named parameter `{}'", std::string::String::from_utf8_lossy(entry.key()));
@@ -429,20 +459,20 @@ macro_rules! params {
             entry.or_insert($crate::value::Value::from($name));
         }
     );
-    (@expand $map:expr;) => {};
-    (@expand $map:expr; $name:expr => $value:expr, $($tail:tt)*) => {
+    (@expand $map:expr_2021;) => {};
+    (@expand $map:expr_2021; $name:expr_2021 => $value:expr_2021, $($tail:tt)*) => {
         params!(@to_pair $map, $name => $value);
         params!(@expand $map; $($tail)*);
     };
-    (@expand $map:expr; $name:expr => $value:expr $(, $tail:tt)*) => {
+    (@expand $map:expr_2021; $name:expr_2021 => $value:expr_2021 $(, $tail:tt)*) => {
         params!(@to_pair $map, $name => $value);
         params!(@expand $map; $($tail)*);
     };
-    (@expand $map:expr; $name:ident, $($tail:tt)*) => {
+    (@expand $map:expr_2021; $name:ident, $($tail:tt)*) => {
         params!(@to_pair $map, $name);
         params!(@expand $map; $($tail)*);
     };
-    (@expand $map:expr; $name:ident $(, $tail:tt)*) => {
+    (@expand $map:expr_2021; $name:ident $(, $tail:tt)*) => {
         params!(@to_pair $map, $name);
         params!(@expand $map; $($tail)*);
     };
@@ -453,7 +483,7 @@ macro_rules! params {
             $crate::params::Params::Named(map)
         }
     };
-    ($i:expr => $($tail:tt)*) => {
+    ($i:expr_2021 => $($tail:tt)*) => {
         {
             let mut map: std::collections::HashMap<std::vec::Vec<u8>, $crate::value::Value, _> = std::default::Default::default();
             params!(@expand (&mut map); $i => $($tail)*);
@@ -569,7 +599,7 @@ fn params_macro_should_panic_on_named_param_redefinition() {
 
 #[test]
 fn issue_88() {
-    use crate::{prelude::FromValue, Value};
+    use crate::{Value, prelude::FromValue};
     #[derive(FromValue, Debug, Eq, PartialEq)]
     #[mysql(is_integer)]
     #[repr(u8)]
@@ -588,7 +618,7 @@ fn issue_88() {
 
 #[test]
 fn from_value_is_string() {
-    use crate::{prelude::FromValue, Value};
+    use crate::{Value, prelude::FromValue};
     #[derive(FromValue, Debug, Eq, PartialEq)]
     #[mysql(is_string, rename_all = "snake_case")]
     enum SomeTypeIsString {
@@ -625,7 +655,7 @@ fn from_value_is_string() {
 
 #[test]
 fn from_value_is_integer() {
-    use crate::{prelude::FromValue, Value};
+    use crate::{Value, prelude::FromValue};
     #[derive(FromValue, Debug, Eq, PartialEq)]
     #[mysql(is_integer, rename_all = "snake_case")]
     #[repr(i8)]
@@ -655,11 +685,11 @@ fn from_value_is_integer() {
 #[cfg(test)]
 mod tests {
     use crate::{
+        FromValueError,
         constants::ColumnType,
         packets::Column,
         row::{convert::FromRow, new_row},
-        value::{convert::from_value, Value},
-        FromValueError,
+        value::{Value, convert::from_value},
     };
     use unic_langid::LanguageIdentifier;
 
@@ -712,7 +742,7 @@ mod tests {
             text_data: String,
             #[mysql(json)]
             json_data: serde_json::Value,
-            #[mysql(with = "from_literal", rename = "custom")]
+            #[mysql(deserialize_with = "from_literal", rename = "custom")]
             custom_bool: bool,
         }
 
