@@ -10,7 +10,7 @@ use bytes::{Buf, BufMut, BytesMut};
 
 use crate::{
     constants::DEFAULT_MAX_ALLOWED_PACKET,
-    proto::codec::{error::PacketCodecError, PacketCodec},
+    proto::codec::{PacketCodec, error::PacketCodecError},
 };
 
 use std::{
@@ -24,7 +24,7 @@ use std::{
 
 // stolen from futures-rs
 macro_rules! with_interrupt {
-    ($e:expr) => {
+    ($e:expr_2021) => {
         loop {
             match $e {
                 Ok(x) => {
